/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2007,2008 Bull S.A.S.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ClientClusterViewCache.java 1597 2008-01-28 13:15:47Z loris $
 * --------------------------------------------------------------------------
 */

package org.ow2.carol.cmi.controller.client;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import javax.ejb.EJBObject;

import net.jcip.annotations.ThreadSafe;

import org.ow2.carol.cmi.lb.data.LBPolicyData;
import org.ow2.carol.cmi.lb.policy.ILBPolicy;
import org.ow2.carol.cmi.lb.strategy.ILBStrategy;
import org.ow2.carol.cmi.reference.CMIReference;
import org.ow2.carol.cmi.reference.ObjectNotFoundException;
import org.ow2.carol.cmi.reference.ServerRef;

/**
 * A cache of cluster view at client side.
 * ClientClusterViewManager refreshes periodically this view.
 * @author The new CMI team
 * @see ClientClusterViewManagerImpl
 */
@ThreadSafe
final class ClientClusterViewCache {

    /**
     * Associations object name - replicated data for policy.
     */
    private final Map<String, LBPolicyData> lbPolicyData =
        new ConcurrentHashMap<String, LBPolicyData>();

    /**
     * Associations object name - name of interface.
     */
    private final Map<String, Class<?>> itfNames =
        new ConcurrentHashMap<String, Class<?>>();

    /**
     * Set of names of replicated objects.
     */
    private final Set<String> replicatedObjects =
        Collections.synchronizedSet(new HashSet<String>());

    /**
     * Set of names of stateful objects.
     */
    private final Set<String> statefulObjects =
        Collections.synchronizedSet(new HashSet<String>());

    /**
     * Associations object name - name of business interface.
     */
    private final Map<String, Class<? extends EJBObject>> businessNames =
        new ConcurrentHashMap<String, Class<? extends EJBObject>>();

    /**
     * Associations object name - type of policy.
     */
    @SuppressWarnings("unchecked")
    private final Map<String, Class<? extends ILBPolicy>> lbPolicyClasses =
        new ConcurrentHashMap<String, Class<? extends ILBPolicy>>();

    /**
     * Associations object name - type of strategy.
     */
    @SuppressWarnings("unchecked")
    private final Map<String, Class<? extends ILBStrategy>> lbStrategyClasses =
        new ConcurrentHashMap<String, Class<? extends ILBStrategy>>();

    /**
     * Associations object name - list of CMI references.
     */
    private final Map<String, List<CMIReference>> cmiReferences =
        new ConcurrentHashMap<String, List<CMIReference>>();

    /**
     * Associations address - load factor.
     */
    private final Map<ServerRef, Integer> loadFactors =
        new ConcurrentHashMap<ServerRef, Integer>();

    /**
     * Associations object name - classnames of the application exceptions.
     */
    private final Map<String, Set<String>> applicationExceptionNames =
        new ConcurrentHashMap<String, Set<String>>();

    /**
     * Default constructor.
     */
    public ClientClusterViewCache() {}

    /**
     * Sets a object's load-balancing policy data (maybe a new version).
     * @param objectName a name of object
     * @param lbPolicyData The load balancing policy data
     */
    void setLBPolicyData(final String objectName, final LBPolicyData lbPolicyData){
        this.lbPolicyData.put(objectName, lbPolicyData);
    }

    /**
     * @param objectName a name of object
     * @return LB policy data for a given object
     */
    LBPolicyData getLBPolicyData(final String objectName) {
        return lbPolicyData.get(objectName);
    }

    /**
     * Sets the interface name of the object with the given name.
     * @param objectName a name of object
     * @param itfName an interface name
     */
    void setItfName(final String objectName, final Class<?> itfName){
        itfNames.put(objectName, itfName);
    }

    /**
     * Gets the interface name of the object with the given name.
     * @param objectName The name of object
     * @return the interface name of the object with the given name
     */
    Class<?> getItfName(final String objectName) {
        return itfNames.get(objectName);
    }

    /**
     * Sets the business interface name of the object with the given name.
     * @param objectName a name of object
     * @param businessName a business interface name
     */
    void setBusinessName(final String objectName, final Class<? extends EJBObject> businessName){
        businessNames.put(objectName, businessName);
    }

    /**
     * Gets the business interface name of the object with the given name.
     * @param objectName a name of object
     * @return the business interface name of the object with the given name
     */
    Class<? extends EJBObject> getBusinessName(final String objectName) {
        return businessNames.get(objectName);
    }

    /**
     * Gets the class of a load-balancing policy.
     * @param className a name of class
     * @return The class of the given load-balancing policy
     */
    @SuppressWarnings("unchecked")
    Class<? extends ILBPolicy> getLBPolicyClass(final String className) {
        return lbPolicyClasses.get(className);
    }

    /**
     * Sets the class for the given name of LB strategy.
     * @param className a name of class
     * @param lbPolicyClass a class for the given name of LB policy
     */
    @SuppressWarnings("unchecked")
    void setLBPolicyClass(final String className, final Class<? extends ILBPolicy> lbPolicyClass) {
        lbPolicyClasses.put(className, lbPolicyClass);
    }

    /**
     * Sets the list of CMIReference of a object with a given name.
     * @param objectName a name of object
     * @param cmiReferences a list of CMIReference
     */
    void setCMIReferences(final String objectName, final List<CMIReference> cmiReferences) {
        this.cmiReferences.put(objectName, cmiReferences);
    }

    /**
     * Returns the list of CMIReference of a object with a given name.
     * @param objectName a name of object
     * @return the list of CMIReference of a object with a given name
     */
    List<CMIReference> getCMIReferences(final String objectName) {
        return cmiReferences.get(objectName);
    }

    /**
     * Returns the class for the given name of LB strategy.
     * @param className a name of class
     * @return the class for the given name of LB strategy
     */
    @SuppressWarnings("unchecked")
    Class<? extends ILBStrategy> getLBStrategyClass(final String className) {
        return lbStrategyClasses.get(className);
    }

    /**
     * Sets the class for the given name of LB strategy.
     * @param className a name of class
     * @param lbStrategyClass a class for the given name of LB strategy
     */
    @SuppressWarnings("unchecked")
    void setLBStrategyClass(
            final String className, final Class<? extends ILBStrategy> lbStrategyClass) {
        lbStrategyClasses.put(className, lbStrategyClass);
    }

    /**
     * Returns the load-factor for the server with the given address.
     * @param serverRef a reference on a server
     * @return the load-factor for the server with the given address
     */
    int getLoadFactor(final ServerRef serverRef) {
        return loadFactors.get(serverRef);
    }

    /**
     * Sets the load-factor for the server with the given address.
     * @param serverRef a reference on a server
     * @param loadFactor the load-factor for the server with the given address
     */
    void setLoadFactor(final ServerRef serverRef, final int loadFactor) {
        loadFactors.put(serverRef, loadFactor);
    }

    /**
     * Returns the set of references on known servers.
     * @return the set of references on known servers
     */
    Set<ServerRef> getAddressesOfServer() {
        return loadFactors.keySet();
    }

    /**
     * @return the set of clustered object names
     */
    Set<String> getObjectNames() {
        return new HashSet<String>(itfNames.keySet());
    }

    /**
     * Return true if the object with the given name is stateful.
     * @param objectName a name of object
     * @return true if the object with the given name is stateful
     */
    boolean hasState(final String objectName) {
        return statefulObjects.contains(objectName);
    }

    /**
     * Set if the object with the given name is stateful.
     * @param objectName a name of object
     */
    void setState(final String objectName) {
        statefulObjects.add(objectName);
    }

    /**
     * Return true if the object with the given name is replicated for high-availability.
     * @param objectName a name of object
     * @return true if the object with the given name is replicated for high-availability
     */
    boolean isReplicated(final String objectName) {
        return replicatedObjects.contains(objectName);
    }

    /**
     * Set if the object with the given name is replicated for high-availability.
     * @param objectName a name of object
     */
    void setReplicated(final String objectName) {
        replicatedObjects.add(objectName);
    }

    /**
     * Return classnames of the application exceptions.
     * @param objectName a name of object
     * @return classnames of the application exceptions
     * @throws ObjectNotFoundException if none object has the given name
     */
    Set<String> getApplicationExceptionNames(final String objectName) {
        return applicationExceptionNames.get(objectName);
    }

    /**
     * Set the classnames of the application exceptions.
     * @param objectName a name of object
     * @param classnames of the application exceptions
     * @throws ObjectNotFoundException if none object has the given name
     */
    void setApplicationExceptionNames(final String objectName, final Set<String> applicationExceptionNames) {
        this.applicationExceptionNames.put(objectName, applicationExceptionNames);
    }

}
